@@ -8,14 +8,13 @@ of frames is implemented in :mod:`websockets.protocol`.
 .. _section 5 of RFC 6455: http://tools.ietf.org/html/rfc6455#section-5
 """
 
+import asyncio
 import collections
 import io
 import random
 import struct
 
-import asyncio
-
-from .exceptions import WebSocketProtocolError, PayloadTooBig
+from .exceptions import PayloadTooBig, WebSocketProtocolError
 
 
 __all__ = [
@@ -44,6 +43,16 @@ CLOSE_CODES = {
 
 
 Frame = collections.namedtuple('Frame', ('fin', 'opcode', 'data'))
+Frame.__doc__ = """WebSocket frame.
+
+* ``fin`` is the FIN bit
+* ``opcode`` is the opcode
+* ``data`` is the payload data
+
+Only these three fields are needed by higher level code. The MASK bit, payload
+length and masking-key are handled on the fly by :func:`read_frame` and
+:func:`write_frame`.
+"""
 
 
 @asyncio.coroutine
@@ -51,17 +60,18 @@ def read_frame(reader, mask, *, max_size=None):
     """
     Read a WebSocket frame and return a :class:`Frame` object.
 
-    `reader` is a coroutine taking an integer argument and reading exactly this
-    number of bytes, unless the end of file is reached.
+    ``reader`` is a coroutine taking an integer argument and reading exactly
+    this number of bytes, unless the end of file is reached.
 
-    `mask` is a :class:`bool` telling whether the frame should be masked, ie.
-    whether the read happens on the server side.
+    ``mask`` is a :class:`bool` telling whether the frame should be masked
+    i.e. whether the read happens on the server side.
 
-    If `max_size` is set and the payload exceeds this size in bytes,
-    :exc:`PayloadTooBig` is raised.
+    If ``max_size`` is set and the payload exceeds this size in bytes,
+    :exc:`~websockets.exceptions.PayloadTooBig` is raised.
 
     This function validates the frame before returning it and raises
-    :exc:`WebSocketProtocolError` if it contains incorrect values.
+    :exc:`~websockets.exceptions.WebSocketProtocolError` if it contains
+    incorrect values.
     """
     # Read the header
     data = yield from reader(2)
@@ -99,46 +109,50 @@ def write_frame(frame, writer, mask):
     """
     Write a WebSocket frame.
 
-    `frame` is the :class:`Frame` object to write.
+    ``frame`` is the :class:`Frame` object to write.
 
-    `writer` is a function accepting bytes.
+    ``writer`` is a function accepting bytes.
 
-    `mask` is a :class:`bool` telling whether the frame should be masked, ie.
-    whether the write happens on the client side.
+    ``mask`` is a :class:`bool` telling whether the frame should be masked
+    i.e. whether the write happens on the client side.
 
     This function validates the frame before sending it and raises
-    :exc:`WebSocketProtocolError` if it contains incorrect values.
+    :exc:`~websockets.exceptions.WebSocketProtocolError` if it contains
+    incorrect values.
     """
     check_frame(frame)
+    output = io.BytesIO()
 
-    # Write the header
-    header = io.BytesIO()
+    # Prepare the header
     head1 = 0b10000000 if frame.fin else 0
     head1 |= frame.opcode
     head2 = 0b10000000 if mask else 0
     length = len(frame.data)
     if length < 0x7e:
-        header.write(struct.pack('!BB', head1, head2 | length))
+        output.write(struct.pack('!BB', head1, head2 | length))
     elif length < 0x10000:
-        header.write(struct.pack('!BBH', head1, head2 | 126, length))
+        output.write(struct.pack('!BBH', head1, head2 | 126, length))
     else:
-        header.write(struct.pack('!BBQ', head1, head2 | 127, length))
+        output.write(struct.pack('!BBQ', head1, head2 | 127, length))
     if mask:
         mask_bits = struct.pack('!I', random.getrandbits(32))
-        header.write(mask_bits)
-    writer(header.getvalue())
+        output.write(mask_bits)
 
-    # Write the data
+    # Prepare the data
     if mask:
         data = bytes(b ^ mask_bits[i % 4] for i, b in enumerate(frame.data))
     else:
         data = frame.data
-    writer(data)
+    output.write(data)
+
+    # Send the frame
+    writer(output.getvalue())
 
 
 def check_frame(frame):
     """
-    Raise :exc:`WebSocketProtocolError` if the frame contains incorrect values.
+    Raise :exc:`~websockets.exceptions.WebSocketProtocolError` if the frame
+    contains incorrect values.
     """
     if frame.opcode in (OP_CONT, OP_TEXT, OP_BINARY):
         return
@@ -155,11 +169,11 @@ def parse_close(data):
     """
     Parse the data in a close frame.
 
-    Return `(code, reason)` when `code` is an :class:`int` and `reason` a
-    :class:`str`.
+    Return ``(code, reason)`` when ``code`` is an :class:`int` and ``reason``
+    a :class:`str`.
 
-    Raise :exc:`WebSocketProtocolError` or :exc:`UnicodeDecodeError` if the
-    data is invalid.
+    Raise :exc:`~websockets.exceptions.WebSocketProtocolError` or
+    :exc:`UnicodeDecodeError` if the data is invalid.
     """
     length = len(data)
     if length == 0:
