@@ -1,7 +1,6 @@
 """
 The :mod:`websockets.http` module provides HTTP parsing functions. They're
-merely adequate for the WebSocket handshake messages. They're used by the
-sample client and servers.
+merely adequate for the WebSocket handshake messages.
 
 These functions cannot be imported from :mod:`websockets`; they must be
 imported from :mod:`websockets.http`.
@@ -9,12 +8,11 @@ imported from :mod:`websockets.http`.
 
 __all__ = ['read_request', 'read_response', 'USER_AGENT']
 
+import asyncio
 import email.parser
 import io
 import sys
 
-import asyncio
-
 from .version import version as websockets_version
 
 
@@ -30,10 +28,11 @@ USER_AGENT = ' '.join((
 @asyncio.coroutine
 def read_request(stream):
     """
-    Read an HTTP/1.1 request from `stream`.
+    Read an HTTP/1.1 request from ``stream``.
 
-    Return `(path, headers)` where `path` is a :class:`str` and `headers` is a
-    :class:`~email.message.Message`; `path` isn't URL-decoded.
+    Return ``(path, headers)`` where ``path`` is a :class:`str` and
+    ``headers`` is a :class:`~email.message.Message`. ``path`` isn't
+    URL-decoded.
 
     Raise an exception if the request isn't well formatted.
 
@@ -51,10 +50,10 @@ def read_request(stream):
 @asyncio.coroutine
 def read_response(stream):
     """
-    Read an HTTP/1.1 response from `stream`.
+    Read an HTTP/1.1 response from ``stream``.
 
-    Return `(status, headers)` where `status` is a :class:`int` and `headers`
-    is a :class:`~email.message.Message`.
+    Return ``(status, headers)`` where ``status`` is a :class:`int` and
+    ``headers`` is a :class:`~email.message.Message`.
 
     Raise an exception if the request isn't well formatted.
 
@@ -70,10 +69,10 @@ def read_response(stream):
 @asyncio.coroutine
 def read_message(stream):
     """
-    Read an HTTP message from `stream`.
+    Read an HTTP message from ``stream``.
 
-    Return `(start_line, headers)` where `start_line` is :class:`bytes` and
-    `headers` is a :class:`~email.message.Message`.
+    Return ``(start_line, headers)`` where ``start_line`` is :class:`bytes`
+    and ``headers`` is a :class:`~email.message.Message`.
 
     The message is assumed not to contain a body.
     """
@@ -94,7 +93,7 @@ def read_message(stream):
 @asyncio.coroutine
 def read_line(stream):
     """
-    Read a single line from `stream`.
+    Read a single line from ``stream``.
     """
     line = yield from stream.readline()
     if len(line) > MAX_LINE:
