@@ -7,15 +7,14 @@ frames as specified in `sections 4 to 8 of RFC 6455`_.
 
 __all__ = ['WebSocketCommonProtocol']
 
+import asyncio
+import asyncio.queues
 import codecs
 import collections
 import logging
 import random
 import struct
 
-import asyncio
-from asyncio.queues import Queue, QueueEmpty
-
 from .exceptions import InvalidState, PayloadTooBig, WebSocketProtocolError
 from .framing import *
 from .handshake import *
@@ -24,6 +23,15 @@ from .handshake import *
 logger = logging.getLogger(__name__)
 
 
+# A WebSocket connection goes through the following four states, in order:
+
+CONNECTING, OPEN, CLOSING, CLOSED = range(4)
+
+# In order to ensure consistency, the code always checks the current value of
+# WebSocketCommonProtocol.state before assigning a new value and never yields
+# between the check and the assignment.
+
+
 class WebSocketCommonProtocol(asyncio.StreamReaderProtocol):
     """
     This class implements common parts of the WebSocket protocol.
@@ -37,19 +45,30 @@ class WebSocketCommonProtocol(asyncio.StreamReaderProtocol):
     control frames automatically. It sends outgoing data frames and performs
     the closing handshake.
 
-    The `host`, `port` and `secure` parameters are simply stored as attributes
-    for handlers that need them.
+    The ``host``, ``port`` and ``secure`` parameters are simply stored as
+    attributes for handlers that need them.
 
-    The `timeout` parameter defines the maximum wait time in seconds for
+    The ``timeout`` parameter defines the maximum wait time in seconds for
     completing the closing handshake and, only on the client side, for
     terminating the TCP connection. :meth:`close()` will complete in at most
     this time on the server side and twice this time on the client side.
 
-    The `max_size` parameter enforces the maximum size for incoming messages
+    The ``max_size`` parameter enforces the maximum size for incoming messages
     in bytes. The default value is 1MB. ``None`` disables the limit. If a
     message larger than the maximum size is received, :meth:`recv()` will
     return ``None`` and the connection will be closed with status code 1009.
 
+    Once the handshake is complete, request and response HTTP headers are
+    available:
+
+    * as a MIME :class:`~email.message.Message` in the :attr:`request_headers`
+      and :attr:`response_headers` attributes
+    * as an iterable of (name, value) pairs in the :attr:`raw_request_headers`
+      and :attr:`raw_response_headers` attributes
+
+    If a subprotocol was negotiated, it's available in the :attr:`subprotocol`
+    attribute.
+
     Once the connection is closed, the status code is available in the
     :attr:`close_code` attribute and the reason in :attr:`close_reason`.
     """
@@ -60,44 +79,91 @@ class WebSocketCommonProtocol(asyncio.StreamReaderProtocol):
     # To get the client-side behavior, set is_client = True.
 
     is_client = False
-    state = 'OPEN'
+    state = OPEN
 
     def __init__(self, *,
-                 host=None, port=None, secure=None, timeout=10, max_size=2 ** 20, loop=None):
+                 host=None, port=None, secure=None,
+                 timeout=10, max_size=2 ** 20, loop=None):
         self.host = host
         self.port = port
         self.secure = secure
 
         self.timeout = timeout
         self.max_size = max_size
+        # Store a reference to loop to avoid relying on self._loop, a private
+        # attribute of StreamReaderProtocol, inherited from FlowControlMixin.
+        self.loop = loop
 
-        super().__init__(asyncio.StreamReader(), self.client_connected, loop)
+        stream_reader = asyncio.StreamReader(loop=loop)
+        super().__init__(stream_reader, self.client_connected, loop)
 
+        self.reader = None
+        self.writer = None
+
+        self.request_headers = None
+        self.raw_request_headers = None
+        self.response_headers = None
+        self.raw_response_headers = None
+
+        self.subprotocol = None
+
+        # Code and reason must be set when the closing handshake completes.
         self.close_code = None
         self.close_reason = ''
 
         # Futures tracking steps in the connection's lifecycle.
-        self.opening_handshake = asyncio.Future()
-        self.closing_handshake = asyncio.Future()
-        self.connection_failed = asyncio.Future()
-        self.connection_closed = asyncio.Future()
+        # Set to True when the opening handshake has completed properly.
+        self.opening_handshake = asyncio.Future(loop=loop)
+        # Set to True when the closing handshake has completed properly and to
+        # False when the connection terminates abnormally.
+        self.closing_handshake = asyncio.Future(loop=loop)
+        # Set to None when the connection state becomes CLOSED.
+        self.connection_closed = asyncio.Future(loop=loop)
 
         # Queue of received messages.
-        self.messages = Queue()
+        self.messages = asyncio.queues.Queue(loop=loop)
 
         # Mapping of ping IDs to waiters, in chronological order.
         self.pings = collections.OrderedDict()
 
-        # Task managing the connection.
-        self.worker = asyncio.async(self.run())
+        # Task managing the connection, initalized in self.client_connected.
+        self.worker = None
 
         # In a subclass implementing the opening handshake, the state will be
         # CONNECTING at this point.
-        if self.state == 'OPEN':
+        if self.state == OPEN:
             self.opening_handshake.set_result(True)
 
+    @property
+    def state_name(self):
+        return ['CONNECTING', 'OPEN', 'CLOSING', 'CLOSED'][self.state]
+
     # Public API
 
+    @property
+    def local_address(self):
+        """
+        Local address of the connection.
+
+        The address is a ``(host, port)`` tuple or ``None`` if the connection
+        hasn't been established yet.
+        """
+        if self.writer is None:
+            return None
+        return self.writer.get_extra_info('sockname')
+
+    @property
+    def remote_address(self):
+        """
+        Remote address of the connection.
+
+        The address is a ``(host, port)`` tuple or ``None`` if the connection
+        hasn't been established yet.
+        """
+        if self.writer is None:
+            return None
+        return self.writer.get_extra_info('peername')
+
     @property
     def open(self):
         """
@@ -105,34 +171,32 @@ class WebSocketCommonProtocol(asyncio.StreamReaderProtocol):
 
         It may be used to handle disconnections gracefully.
         """
-        return self.state == 'OPEN'
+        return self.state == OPEN
 
     @asyncio.coroutine
     def close(self, code=1000, reason=''):
         """
         This coroutine performs the closing handshake.
 
-        This is the expected way to terminate a connection on the server side.
-
         It waits for the other end to complete the handshake. It doesn't do
         anything once the connection is closed.
 
-        It's usually safe to wrap this coroutine in `asyncio.async()` since
-        errors during connection termination aren't particularly useful.
+        It's usually safe to wrap this coroutine in :func:`~asyncio.async`
+        since errors during connection termination aren't particularly useful.
 
-        The `code` must be an :class:`int` and the `reason` a :class:`str`.
+        ``code`` must be an :class:`int` and ``reason`` a :class:`str`.
         """
-        if self.state == 'OPEN':
+        if self.state == OPEN:
             # 7.1.2. Start the WebSocket Closing Handshake
-            self.close_code, self.close_reason = code, reason
-            yield from self.write_frame(OP_CLOSE, serialize_close(code, reason))
             # 7.1.3. The WebSocket Closing Handshake is Started
-            self.state = 'CLOSING'
+            frame_data = serialize_close(code, reason)
+            yield from self.write_frame(OP_CLOSE, frame_data)
 
         # If the connection doesn't terminate within the timeout, break out of
         # the worker loop.
         try:
-            yield from asyncio.wait_for(self.worker, timeout=self.timeout)
+            yield from asyncio.wait_for(
+                self.worker, self.timeout, loop=self.loop)
         except asyncio.TimeoutError:
             self.worker.cancel()
 
@@ -154,14 +218,20 @@ class WebSocketCommonProtocol(asyncio.StreamReaderProtocol):
         # Return any available message
         try:
             return self.messages.get_nowait()
-        except QueueEmpty:
+        except asyncio.queues.QueueEmpty:
             pass
 
         # Wait for a message until the connection is closed
-        next_message = asyncio.async(self.messages.get())
-        done, pending = yield from asyncio.wait(
+        next_message = asyncio.async(self.messages.get(), loop=self.loop)
+        try:
+            done, pending = yield from asyncio.wait(
                 [next_message, self.worker],
-                return_when=asyncio.FIRST_COMPLETED)
+                loop=self.loop, return_when=asyncio.FIRST_COMPLETED)
+        except asyncio.CancelledError:
+            # Handle the Task.cancel()
+            next_message.cancel()
+            raise
+
         if next_message in done:
             return next_message.result()
         else:
@@ -176,7 +246,8 @@ class WebSocketCommonProtocol(asyncio.StreamReaderProtocol):
         frame.
 
         It raises a :exc:`TypeError` for other inputs and
-        :exc:`InvalidState` once the connection is closed.
+        :exc:`~websockets.exceptions.InvalidState` once the connection is
+        closed.
         """
         if isinstance(data, str):
             opcode = 1
@@ -192,8 +263,9 @@ class WebSocketCommonProtocol(asyncio.StreamReaderProtocol):
         """
         This coroutine sends a ping.
 
-        It returns a Future which will be completed when the corresponding
-        pong is received and which you may ignore if you don't want to wait.
+        It returns a :class:`~asyncio.Future` which will be completed when the
+        corresponding pong is received and which you may ignore if you don't
+        want to wait.
 
         A ping may serve as a keepalive.
         """
@@ -204,7 +276,7 @@ class WebSocketCommonProtocol(asyncio.StreamReaderProtocol):
         while data is None or data in self.pings:
             data = struct.pack('!I', random.getrandbits(32))
 
-        self.pings[data] = asyncio.Future()
+        self.pings[data] = asyncio.Future(loop=self.loop)
         yield from self.write_frame(OP_PING, data)
         return self.pings[data]
 
@@ -305,12 +377,13 @@ class WebSocketCommonProtocol(asyncio.StreamReaderProtocol):
             frame = yield from self.read_frame(max_size)
             # 5.5. Control Frames
             if frame.opcode == OP_CLOSE:
-                self.close_code, self.close_reason = parse_close(frame.data)
-                if self.state != 'CLOSING':
+                # Make sure the close frame is valid before echoing it.
+                code, reason = parse_close(frame.data)
+                if self.state == OPEN:
                     # 7.1.3. The WebSocket Closing Handshake is Started
-                    self.state = 'CLOSING'
-                    yield from self.write_frame(OP_CLOSE, frame.data, 'CLOSING')
+                    yield from self.write_frame(OP_CLOSE, frame.data)
                 if not self.closing_handshake.done():
+                    self.close_code, self.close_reason = code, reason
                     self.closing_handshake.set_result(True)
                 return
             elif frame.opcode == OP_PING:
@@ -332,17 +405,22 @@ class WebSocketCommonProtocol(asyncio.StreamReaderProtocol):
     @asyncio.coroutine
     def read_frame(self, max_size):
         is_masked = not self.is_client
-        frame = yield from read_frame(self.reader.readexactly, is_masked, max_size=max_size)
+        frame = yield from read_frame(
+            self.reader.readexactly, is_masked, max_size=max_size)
         side = 'client' if self.is_client else 'server'
         logger.debug("%s << %s", side, frame)
         return frame
 
     @asyncio.coroutine
-    def write_frame(self, opcode, data=b'', expected_state='OPEN'):
+    def write_frame(self, opcode, data=b''):
         # This may happen if a user attempts to write on a closed connection.
-        if self.state != expected_state:
+        if self.state != OPEN:
             raise InvalidState("Cannot write to a WebSocket "
-                               "in the {} state".format(self.state))
+                               "in the {} state".format(self.state_name))
+        # Make sure no other frame will be sent after a close frame. Do this
+        # before yielding control to avoid sending more than one close frame.
+        if opcode == OP_CLOSE:
+            self.state = CLOSING
         frame = Frame(True, opcode, data)
         side = 'client' if self.is_client else 'server'
         logger.debug("%s >> %s", side, frame)
@@ -352,70 +430,60 @@ class WebSocketCommonProtocol(asyncio.StreamReaderProtocol):
             # Handle flow control automatically.
             yield from self.writer.drain()
         except ConnectionResetError:
-            # Terminate the connection if the socket died,
-            # unless it's already being closed.
-            if expected_state != 'CLOSING':
-                self.state = 'CLOSING'
-                yield from self.fail_connection(1006)
+            # Terminate the connection if the socket died.
+            yield from self.fail_connection(1006)
 
     @asyncio.coroutine
-    def close_connection(self):
+    def close_connection(self, force=False):
         # 7.1.1. Close the WebSocket Connection
-        if self.state == 'CLOSED':
+        if self.state == CLOSED:
             return
 
         # Defensive assertion for protocol compliance.
-        if self.state != 'CLOSING':                         # pragma: no cover
+        if self.state != CLOSING and not force:             # pragma: no cover
             raise InvalidState("Cannot close a WebSocket connection "
-                               "in the {} state".format(self.state))
+                               "in the {} state".format(self.state_name))
 
-        if self.is_client:
+        if self.is_client and not force:
             try:
-                yield from asyncio.wait_for(self.connection_closed,
-                        timeout=self.timeout)
+                yield from asyncio.wait_for(
+                    self.connection_closed, self.timeout, loop=self.loop)
             except (asyncio.CancelledError, asyncio.TimeoutError):
                 pass
 
-            if self.state == 'CLOSED':
+            if self.state == CLOSED:
                 return
 
         # Attempt to terminate the TCP connection properly.
-        # If the socket is already closed, this will crash.
+        # If the socket is already closed, this may crash.
         try:
             if self.writer.can_write_eof():
                 self.writer.write_eof()
-        except Exception:
+        except Exception:                                   # pragma: no cover
             pass
 
         self.writer.close()
 
         try:
-            yield from asyncio.wait_for(self.connection_closed,
-                    timeout=self.timeout)
+            yield from asyncio.wait_for(
+                self.connection_closed, self.timeout, loop=self.loop)
         except (asyncio.CancelledError, asyncio.TimeoutError):
             pass
 
     @asyncio.coroutine
     def fail_connection(self, code=1011, reason=''):
-        # Avoid calling fail_connection more than once to minimize
-        # the consequences of race conditions between the two sides.
-        if self.connection_failed.done():
-            # Wait until the other coroutine calls connection_lost.
-            yield from self.connection_closed
-            return
-        else:
-            self.connection_failed.set_result(None)
-
-        # Losing the connection usually results in a protocol error.
-        # Preserve the original error code in this case.
-        if self.close_code != 1006:
-            self.close_code, self.close_reason = code, reason
         # 7.1.7. Fail the WebSocket Connection
         logger.info("Failing the WebSocket connection: %d %s", code, reason)
-        if self.state == 'OPEN':
-            yield from self.write_frame(OP_CLOSE, serialize_close(code, reason))
-            self.state = 'CLOSING'
+        if self.state == OPEN:
+            if code == 1006:
+                # Don't send a close frame is the connection is broken. Set
+                # the state to CLOSING to allow close_connection to proceed.
+                self.state = CLOSING
+            else:
+                frame_data = serialize_close(code, reason)
+                yield from self.write_frame(OP_CLOSE, frame_data)
         if not self.closing_handshake.done():
+            self.close_code, self.close_reason = code, reason
             self.closing_handshake.set_result(False)
         yield from self.close_connection()
 
@@ -424,12 +492,15 @@ class WebSocketCommonProtocol(asyncio.StreamReaderProtocol):
     def client_connected(self, reader, writer):
         self.reader = reader
         self.writer = writer
+        # Start the task that handles incoming messages.
+        self.worker = asyncio.async(self.run(), loop=self.loop)
 
     def connection_lost(self, exc):
         # 7.1.4. The WebSocket Connection is Closed
-        self.state = 'CLOSED'
+        self.state = CLOSED
+        if not self.closing_handshake.done():
+            self.close_code, self.close_reason = 1006, ''
+            self.closing_handshake.set_result(False)
         if not self.connection_closed.done():
             self.connection_closed.set_result(None)
-        if self.close_code is None:
-            self.close_code = 1006
         super().connection_lost(exc)
